package com.rinkai.map.api;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;

import javax.imageio.ImageIO;

import com.rinkai.simpleopt.vlatest.service.AddressWithId;
import com.rinkai.simpleopt.vlatest.service.Coordinates;
import com.rinkai.simpleopt.vlatest.service.Depot;
import com.rinkai.simpleopt.vlatest.service.ISimpleOptFacade;
import com.rinkai.simpleopt.vlatest.service.Location;
import com.rinkai.simpleopt.vlatest.service.LocationAndOrder;
import com.rinkai.simpleopt.vlatest.service.LocationInfo;
import com.rinkai.simpleopt.vlatest.service.MapRequest;
import com.rinkai.simpleopt.vlatest.service.MapRequest.Pois;
import com.rinkai.simpleopt.vlatest.service.MapRequest.Traces;
import com.rinkai.simpleopt.vlatest.service.MapRequestGeometryAroundArea;
import com.rinkai.simpleopt.vlatest.service.MapResponse;
import com.rinkai.simpleopt.vlatest.service.Point2PointRoute;
import com.rinkai.simpleopt.vlatest.service.Route;
import com.rinkai.simpleopt.vlatest.service.SimpleOptFacadeCxfVLatestImplService;
import com.rinkai.simpleopt.vlatest.service.TimeOfDay;
import com.rinkai.simpleopt.vlatest.service.Trace;
import com.rinkai.simpleopt.vlatest.service.Trace.CoordinatesPolylines;
import com.rinkai.simpleopt.vlatest.service.VrpRequest;
import com.rinkai.simpleopt.vlatest.service.VrpRequest.LocationsAndOrders;
import com.rinkai.simpleopt.vlatest.service.VrpResponse;

/**
 * Sample and simple usage of Rinkai's map API
 *
 */
public class Sample {
	static String APIKEY = "test4/soap-sample";
	ISimpleOptFacade facade;

	public Sample() throws MalformedURLException {
		SimpleOptFacadeCxfVLatestImplService service;
		service = new SimpleOptFacadeCxfVLatestImplService();
		facade = service.getSimpleOptFacadeService();
	}

	// -- model factory method - most of them only populate the basic arguments
	// ----------------------------

	public static TimeOfDay makeTimeOfDay(int hour, int min, int sec) {
		TimeOfDay retVal = new TimeOfDay();
		retVal.setHour(hour);
		retVal.setMinutes(min);
		retVal.setSeconds(sec);
		return retVal;
	}

	public static Coordinates makeCoordinates(double lat, double lon) {
		Coordinates retVal = new Coordinates();
		retVal.setX(lon);
		retVal.setY(lat);
		return retVal;
	}

	public static Coordinates withLabel(Coordinates a, String label) {
		Coordinates retVal = new Coordinates();
		retVal.setX(a.getX());
		retVal.setY(a.getY());
		retVal.setLabel(label);
		return retVal;
	}

	public static LocationInfo makeLocationInfo(Location location) {
		LocationInfo retVal = new LocationInfo();
		retVal.setLocation(location);
		return retVal;
	}

	public static Location makeLocation(AddressWithId address, Coordinates coordinates) {
		Location retVal = new Location();
		retVal.setAddress(address);
		retVal.setCoordinates(coordinates);
		return retVal;
	}

	public static AddressWithId makeAddress(String houseNumber, String street, String city, String postCode,
			String country) {
		AddressWithId retVal = new AddressWithId();
		retVal.setHouseNumber(houseNumber);
		retVal.setStreet(street);
		retVal.setCity(city);
		retVal.setPostCode(postCode);
		retVal.setCountry(country);

		return retVal;
	}

	/**
	 * Helper to decode a byte array into an awt Image
	 */
	public static BufferedImage getImageFromBytes(byte[] bytes) throws IOException {

		try (ByteArrayInputStream bais = new ByteArrayInputStream(bytes)) {
			BufferedImage image = ImageIO.read(bais);
			return image;
		}

	}

	public void testSimplestOpt() throws IOException {

		// 1) define the Vrp request - this is the same as the simplest json sample

		// note that addresses are populated just for labelling the map,
		// as the locations are already geocoded
		VrpRequest vrpRequest = new VrpRequest();
		vrpRequest.setLocationsAndOrders(new LocationsAndOrders());
		{
			LocationAndOrder locationAndOrder = new LocationAndOrder();
			locationAndOrder.setLocationInfo(
					makeLocationInfo(makeLocation(makeAddress(null, null, "Velemyšleves", "43801", "cz"),
							makeCoordinates(50.4025882579593, 13.5636822222738))));
			vrpRequest.getLocationsAndOrders().getLocationAndOrder().add(locationAndOrder);
		}

		{
			LocationAndOrder locationAndOrder = new LocationAndOrder();
			locationAndOrder.setLocationInfo(
					makeLocationInfo(makeLocation(makeAddress(null, "Nákladní", "Žatec", "43801", "cz"),
							makeCoordinates(50.3294782459803, 13.5431322189068))));
			vrpRequest.getLocationsAndOrders().getLocationAndOrder().add(locationAndOrder);
		}

		{
			LocationAndOrder locationAndOrder = new LocationAndOrder();
			locationAndOrder.setLocationInfo(
					makeLocationInfo(makeLocation(makeAddress("2281", "Suzdalské nám.", "Louny", "44001", "cz"),
							makeCoordinates(50.3569882504888, 13.7914225958854))));
			vrpRequest.getLocationsAndOrders().getLocationAndOrder().add(locationAndOrder);
		}

		Depot depot = new Depot();
		depot.setLocation(makeLocation(null, makeCoordinates(50.5146082763126, 13.6438122354029)));
		vrpRequest.setDepot(depot);

		vrpRequest.setDetailedTraces(true);

		VrpResponse vrpResponse = facade.getOptimization(APIKEY, vrpRequest);

		System.err.println("generalError:" + vrpResponse.getGeneralError());
		System.err.println("invalidLocations:" + vrpResponse.getInvalidLocations());

		// 2) now draw a map with laos

		MapRequest mapRequest = new MapRequest();

		// specify the area that the map should cover
		MapRequestGeometryAroundArea mapRequestGeometryAroundArea = new MapRequestGeometryAroundArea();
		mapRequestGeometryAroundArea.setArea(vrpResponse.getPlan().getBbox());

		// size of the image
		mapRequestGeometryAroundArea.setViewportHeight(512);
		mapRequestGeometryAroundArea.setViewportWidth(512);
		mapRequest.setGeometryAroundArea(mapRequestGeometryAroundArea);

		// add POIs depot + laos
		mapRequest.setPois(new Pois());
		mapRequest.getPois().getCoordinates()
				.add(withLabel(vrpResponse.getRequest().getDepot().getLocation().getCoordinates(), "Depot"));
		for (LocationAndOrder lao : vrpResponse.getRequest().getLocationsAndOrders().getLocationAndOrder()) {
			mapRequest.getPois().getCoordinates().add(withLabel(lao.getLocationInfo().getLocation().getCoordinates(),
					lao.getLocationInfo().getLocation().getAddress().getCity()));
		}

		mapRequest.setTraces(new Traces());
		// add routes - note that we are not decoding the polylines, just handing them
		// over to the map request
		for (Route route : vrpResponse.getPlan().getRoutes().getRoute()) {
			Trace trace = new Trace();
			trace.setCoordinatesPolylines(new CoordinatesPolylines());
			mapRequest.getTraces().getTrace().add(trace);
			for (Point2PointRoute point2PointRoute : route.getLegs().getLeg()) {
				trace.getCoordinatesPolylines().getString().add(point2PointRoute.getCoordinatesPolyline());
			}
		}

		MapResponse response = facade.getMap(APIKEY, mapRequest);

		BufferedImage mapImage = getImageFromBytes(response.getImageBytes());
		System.err.println("map saved in " + "target/vrp.png");
		ImageIO.write(mapImage, "png", new File("target/vrp.png"));

	}

	public static void main(String... args) throws IOException {
		System.err.println("ride on...");
		Sample sample = new Sample();
		sample.testSimplestOpt();

	}
}
